/*
 *  Clock.cpp -  Clock
 *               Written by Tinic Urou
 *               Public domain
 */

#include <AppKit.h>
#include <InterfaceKit.h>
#include <StorageKit.h>
#include <KernelKit.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <malloc.h>
#include <time.h>


// Types
typedef unsigned char 	UBYTE;
typedef unsigned short 	UWORD;
typedef unsigned long 	ULONG;
typedef char 			BYTE;
typedef short 			WORD;
typedef long 			LONG;

// Constants

const 	ULONG 			APP_SIGNATURE = 'CLCK';
const 	LONG 			DISPLAY_X = 80;
const 	LONG 			DISPLAY_Y = 84;		

extern	UBYTE 			image[];
		system_info		si;
		char			id[256];
			
// Window/Application messages

class 					ClockWindow;
class					ClockView;
class					MainView;
class					StatView;
class					VolView;

// Application object
class Clock : public BApplication 
{
	public:
						Clock();
			void 		ReadyToRun(void);
			bool 		QuitRequested(void);
			void 		AboutRequested(void);

	private:
	
			ClockWindow	*the_window;
};

// Window object
class ClockWindow : public BWindow 
{
	public:
	
						ClockWindow(int width, int height);
			bool 		QuitRequested(void);
			void 		MessageReceived(BMessage *msg);
			void		WorkspaceActivated(long ws, bool state);

	private:
	
			MainView	*main_view;	
			screen_info	si;
};

// View object

class MainView : public BView 
{
	public:

						MainView(BRect frame);
						~MainView();
		 	void 		Draw(BRect update);
			void		Pulse(void);
			
			BBitmap		*clock_bitmap;
			ClockView	*clock_view;
			StatView	*teams,*machine;
			VolView		*vol,*mem;
			uchar		grey;
};

class ClockView : public BView 
{
	public:

						ClockView(BRect frame);
						~ClockView();
		 	void 		Draw(BRect update);
			
			BBitmap		*numbers[10];
			uchar		todo[6];
};


class StatView : public BView 
{
	public:

						StatView(BRect frame);
		 	void 		Draw(BRect update);
			
			long		max;
			long		act;
			long		mod;
			rgb_color	col;
			BRect		bd;
			char		*str;
};

class VolView : public BView 
{
	public:

						VolView(BRect frame);
		 	void 		Draw(BRect update);
			
			float		max;
			float		act;
			float		mod;
			rgb_color	col;
			BRect		bd;
			char		*str;
};

// Create application object and start it
int main(int argc, char **argv)
{	
	Clock *the_app;

	the_app = new Clock();
	the_app->Run();
	delete the_app;

	return 0;
}


/*
 *  Application constructor
 */

Clock::Clock() : BApplication(APP_SIGNATURE)
{
	the_window = NULL;
}

/*
 *	Thats from me!
 */

void Clock::AboutRequested(void) 
{
	char str[256];
	sprintf(str, "DIGIClock by Tinic Urou\n<5uro@informatik.uni-hamburg.de>\nFreely distributable.");
	BAlert *the_alert = new BAlert("", str, "OK");
	the_alert->Go();
}


/*
 *  Open window and start rendering
 */

void Clock::ReadyToRun(void)
{
	// Open window
	the_window = new ClockWindow(DISPLAY_X, DISPLAY_Y);
}


/*
 *  Quit requested (either by menu or by closing the window)
 */

bool Clock::QuitRequested(void)
{
	// Make sure that the window closes first
	if (BApplication::QuitRequested()) 
	{
		return TRUE;
	}
	return FALSE;
}


/*
 *  Window constructor
 */

ClockWindow::ClockWindow(int width, int height) : BWindow(BRect(0, 0, width-1, height-1), "Clock by Tinic", B_BORDERED_WINDOW, B_NOT_RESIZABLE | B_NOT_ZOOMABLE,B_ALL_WORKSPACES)
{
	// Move window to right position
	
	get_screen_info(&si);
	MoveTo(si.frame.Width()-DISPLAY_X,1);

	// Create bitmap view
	Lock();
	main_view = new MainView(BRect(0, 0, DISPLAY_X-1, DISPLAY_Y-1));
	AddChild(main_view);
	Unlock();

	SetPulseRate(1000000);

	// Show the window
	Show();
}


/*
 *  Closing the window quits the program
 */

bool ClockWindow::QuitRequested(void)
{
	be_app->PostMessage(B_QUIT_REQUESTED);
	return TRUE;
}


/*
 *  Handles redraw messages
 */

void ClockWindow::MessageReceived(BMessage *msg)
{
	switch (msg->what) 
	{
		default:
			BWindow::MessageReceived(msg);
	}
}

void ClockWindow::WorkspaceActivated(long ws, bool state)
{
	get_screen_info(&si);
	MoveTo(si.frame.Width()-DISPLAY_X,1);
}

MainView::MainView(BRect frame) : BView(frame, "", B_FOLLOW_NONE, B_WILL_DRAW | B_PULSE_NEEDED)
{
	grey=index_for_color(200,200,200);
	clock_bitmap = new BBitmap(BRect(0, 0, DISPLAY_X-1, DISPLAY_Y-1),B_COLOR_8_BIT,TRUE);
	clock_bitmap->Lock();
	clock_view = new ClockView(BRect(0, 0, DISPLAY_X-1, DISPLAY_Y-1));
	clock_bitmap->AddChild(clock_view);
	teams = new StatView(BRect(4, 34, DISPLAY_X-5, 44));
	get_system_info(&si);
	teams->max=si.max_teams;
	teams->act=si.used_teams;
	teams->col.red=255;
	teams->col.green=64;
	teams->col.blue=0;
	teams->col.alpha=0;
	teams->str="%d teams";

	machine = new StatView(BRect(4, 46, DISPLAY_X-5, 56));
	machine->max=si.max_threads;
	machine->act=si.used_threads;
	machine->col.red=255;
	machine->col.green=64;
	machine->col.blue=0;
	machine->col.alpha=0;
	machine->str="%d threads";

	vol = new VolView(BRect(4, 58, DISPLAY_X-5, 68));
	vol->max=boot_volume().Capacity();
	vol->act=boot_volume().Capacity()-boot_volume().FreeBytes();
	vol->col.red=32;
	vol->col.green=128;
	vol->col.blue=255;
	vol->col.alpha=0;
	vol->str="%.1fMB HD";

	mem = new VolView(BRect(4, 70, DISPLAY_X-5, 80));
	mem->max=(si.max_pages)*4096;
	mem->act=(si.used_pages)*4096;
	mem->col.red=32;
	mem->col.green=128;
	mem->col.blue=255;
	mem->col.alpha=0;
	mem->str="%.1fMB RAM";
	
	clock_bitmap->AddChild(teams);
	clock_bitmap->AddChild(machine);
	clock_bitmap->AddChild(vol);
	clock_bitmap->AddChild(mem);
	clock_bitmap->Unlock();
}

MainView::~MainView()
{
	delete clock_bitmap;
}

void MainView::Pulse()
{
	memset(clock_bitmap->Bits(),grey,32*DISPLAY_X);
	clock_view->Window()->Lock();

	get_system_info(&si);

	teams->max=si.max_teams;
	teams->act=si.used_teams;

	// I should integrate this into the object, I know...

	if(teams->mod!=teams->act)
	{
		teams->Draw(Frame());
		teams->mod=teams->act;
	}
	
	machine->max=si.max_threads;
	machine->act=si.used_threads;
	if(machine->mod!=machine->act)
	{
		machine->Draw(Frame());
		machine->mod=machine->act;
	}
		
	vol->max=boot_volume().Capacity();
	vol->act=boot_volume().Capacity()-boot_volume().FreeBytes();
	if(vol->mod!=vol->act)
	{
		vol->Draw(Frame());
		vol->mod=vol->act;
	}	
	
	mem->max=(si.max_pages)*4096;
	mem->act=(si.used_pages)*4096;
	if(mem->mod!=mem->act)
	{
		mem->Draw(Frame());
		mem->mod=mem->act;
	}	
	
	clock_view->Draw(Frame());
	clock_view->Window()->Unlock();
	Window()->Lock();
	DrawBitmap(clock_bitmap,BPoint(0,0));
	Window()->Unlock();
}

void MainView::Draw(BRect updateRect)
{
	memset(clock_bitmap->Bits(),grey,DISPLAY_Y*DISPLAY_X);
	clock_view->Window()->Lock();

	teams->Draw(updateRect);

	machine->Draw(Frame());

	vol->Draw(Frame());

	mem->Draw(Frame());

	clock_view->Draw(updateRect);
	clock_view->Window()->Unlock();
	DrawBitmap(clock_bitmap);
}

ClockView::ClockView(BRect frame) : BView(frame, "", B_FOLLOW_NONE, B_WILL_DRAW | B_PULSE_NEEDED)
{
	for(int i=0;i<10;i++)
	{
		numbers[i]= new BBitmap(BRect(0,0,11,16),B_COLOR_8_BIT);
		memcpy(numbers[i]->Bits(),&image[i*12*17],12*17);
	}
	SetFontName("Kate");
	SetFontSize(9);
	SetDrawingMode(B_OP_OVER);
}

ClockView::~ClockView()
{
	for(int i=0;i<10;i++)
	{
		if(numbers[i])delete numbers[i];
	}
}

void ClockView::Draw(BRect update)
{
	struct tm *my_tm;
	time_t	my_time;
	SetHighColor(255,255,255);
	FillRect(BRect(Frame().RightTop(),Frame().RightBottom()));	
	FillRect(BRect(Frame().LeftBottom(),Frame().RightBottom()));	
	SetHighColor(0,0,0);
	FillRect(BRect(Frame().LeftTop(),Frame().RightTop()));	
	FillRect(BRect(Frame().LeftTop(),Frame().LeftBottom()));
	my_time=__get_time();
	my_tm=localtime(&my_time);
	todo[0]=my_tm->tm_hour/10;
	todo[1]=my_tm->tm_hour-(todo[0]*10);
	todo[2]=my_tm->tm_min/10;
	todo[3]=my_tm->tm_min-(todo[2]*10);
	todo[4]=my_tm->tm_sec/10;
	todo[5]=my_tm->tm_sec-(todo[4]*10);
	if(my_tm->tm_sec&0x01)
	{
		FillRect(BRect(30,8,31,9));
		FillRect(BRect(30,13,31,14));
	}
	DrawBitmapAsync(numbers[todo[0]],BPoint(6,3));
	DrawBitmapAsync(numbers[todo[1]],BPoint(18,3));
	DrawBitmapAsync(numbers[todo[2]],BPoint(35,3));
	DrawBitmapAsync(numbers[todo[3]],BPoint(47,3));		
	DrawChar(todo[4]+0x30,BPoint(62,20));
	DrawChar(todo[5]+0x30,BPoint(69,20));
	DrawString(ctime(&my_time),BPoint(6,31));
	Sync();
}

StatView::StatView(BRect frame) : BView(frame, "", B_FOLLOW_NONE, B_WILL_DRAW)
{
	bd=Bounds();
	SetFontName("Erich");
	SetFontSize(9);
	SetDrawingMode(B_OP_OVER);
}

void StatView::Draw(BRect update)
{
	char buf[256];
	SetHighColor(255,255,255);
	FillRect(bd);
	SetHighColor(0,0,0);
	StrokeRect(bd);
	SetHighColor(col);
	FillRect(BRect(1,1,(act*(bd.Width()-1))/max,bd.Height()-1));
	sprintf(buf,str,act);
	SetHighColor(0,0,0);
	DrawString(buf,BPoint((bd.Width()-StringWidth(buf))/2,9));
}

VolView::VolView(BRect frame) : BView(frame, "", B_FOLLOW_NONE, B_WILL_DRAW)
{
	bd=Bounds();
	SetFontName("Erich");
	SetFontSize(9);
	SetDrawingMode(B_OP_OVER);
}

void VolView::Draw(BRect update)
{
	char buf[256];
	SetHighColor(255,255,255);
	FillRect(bd);
	SetHighColor(0,0,0);
	StrokeRect(bd);
	SetHighColor(col);
	FillRect(BRect(1,1,(act*(bd.Width()-1))/max,bd.Height()-1));
	sprintf(buf,str,(max-act)/(1024*1024));
	SetHighColor(0,0,0);
	DrawString(buf,BPoint((bd.Width()-StringWidth(buf))/2,9));
}